package io.agora.signal;

import org.junit.Test;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

/**
 * Created by aa on 13/4/2017.
 */
public class SignalTestChannel {
    public static final String appid = "place appid to here";

    String user1 = "test_channel_user1";
    String user2 = "test_channel_user2";

    public static void wait_time(CountDownLatch x, int tInMS){
        try {
            x.await(tInMS, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
        }
        assertEquals(x.getCount(), 0);
    }

    /**
     * test :
     *      API login
     *      API logout
     *      API channelJoin
     *      API channelLeave
     *      API channelSetAttr
     *      API channelDelAttr
     *      API channelClearAttr
     *      API messageChannelSend
     *      API channelQueryUserNum
     *
     *          onLoginSuccess
     *          onLogout
     *
     *          onChannelJoined
     *          onChannelLeaved
     *          onChannelUserJoined
     *          onChannelUserLeaved
     *          onChannelUserList
     *          onChannelAttrUpdated
     *          onChannelQueryUserNumResult
     *          onMessageChannelReceive
     */
    @Test
    public void testChannel(){
        System.out.println();
        System.out.println();

        final String channelName = "test_channel_name";
        Signal sig = new Signal(appid);

        final CountDownLatch loginLatch1 = new CountDownLatch(1);
        final CountDownLatch logoutLatch1 = new CountDownLatch(1);
        final CountDownLatch joinLatch1 = new CountDownLatch(1);
        final CountDownLatch joinListLatch1 = new CountDownLatch(1);

        final CountDownLatch loginLatch2 = new CountDownLatch(1);
        final CountDownLatch logoutLatch2 = new CountDownLatch(1);
        final CountDownLatch joinLatch2 = new CountDownLatch(1);
        final CountDownLatch joinListLatch2 = new CountDownLatch(1);
        final CountDownLatch joinAttrLatch2 = new CountDownLatch(1);

        final CountDownLatch attrSet_name_Latch = new CountDownLatch(1);
        final CountDownLatch attrSetLatch = new CountDownLatch(2);
        final CountDownLatch attrDelLatch = new CountDownLatch(2);
        final CountDownLatch attrClearLatch = new CountDownLatch(2);

        final CountDownLatch channelMsgLatch1 = new CountDownLatch(3);

        final CountDownLatch userJoinLatch = new CountDownLatch(1);
        final CountDownLatch userLeaveLatch = new CountDownLatch(1);

        Signal.LoginSession s1 = sig.login(user1, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch1.countDown();
                session.channelJoin(channelName, new Signal.ChannelCallback(){
                    @Override
                    public void onChannelJoined(Signal.LoginSession session, Signal.LoginSession.Channel channel) {
                        joinLatch1.countDown();
                        channel.channelSetAttr("name", "world");
                        channel.messageChannelSend("msg1");
                    }

                    @Override
                    public void onChannelUserList(Signal.LoginSession session, Signal.LoginSession.Channel channel, List<String> users, List<Integer> uids) {
                        joinListLatch1.countDown();
                        assertEquals(users.size(), 1);
                        assertEquals(users.get(0), user1);
                    }

                    @Override
                    public void onMessageChannelReceive(Signal.LoginSession session,  Signal.LoginSession.Channel channel, String account, int uid, String msg) {
                        if (msg.equals("msg1")){
                            channelMsgLatch1.countDown();
                        }
                    }

                    @Override
                    public void onChannelUserJoined(Signal.LoginSession session, Signal.LoginSession.Channel channel, String account, int uid) {
                        userJoinLatch.countDown();

                        channel.channelSetAttr("name2", "bbb");
                        channel.channelDelAttr("name2");
                        channel.channelClearAttr();
                    }

                    @Override
                    public void onChannelUserLeaved(Signal.LoginSession session, Signal.LoginSession.Channel channel, String account, int uid) {
                        userLeaveLatch.countDown();

                        session.logout();
                    }

                    @Override
                    public void onChannelAttrUpdated(Signal.LoginSession session, Signal.LoginSession.Channel channel, String name, String value, String type) {
                        if (type.equals("update") && name.equals("name")){
                            attrSet_name_Latch.countDown();
                        }

                        if (type.equals("update") && name.equals("name2")){
                            attrSetLatch.countDown();
                        }

                        if (type.equals("del")){
                            attrDelLatch.countDown();
                        }

                        if (type.equals("clear")){
                            attrClearLatch.countDown();
                        }
                    }
                });
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch1.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }
        });


        wait_time(loginLatch1, 5000);
        wait_time(joinLatch1, 5000);
        wait_time(joinListLatch1, 5000);

        Signal.LoginSession s2 = sig.login(user2, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch2.countDown();

                session.channelJoin(channelName, new Signal.ChannelCallback(){
                    @Override
                    public void onChannelJoined(Signal.LoginSession session, Signal.LoginSession.Channel channel) {
                        joinLatch2.countDown();
                    }

                    @Override
                    public void onChannelUserList(Signal.LoginSession session, Signal.LoginSession.Channel channel, List<String> users, List<Integer> uids) {
                        joinListLatch2.countDown();
                        assertEquals(users.size(), 2);
                    }

                    @Override
                    public void onChannelAttrUpdated(Signal.LoginSession session, Signal.LoginSession.Channel channel, String name, String value, String type) {
                        if (type.equals("update") && name.equals("name")){
                            joinAttrLatch2.countDown();
                            assertEquals(value, "world");

                            channel.messageChannelSend("msg1");
                        }

                        if (type.equals("update") && name.equals("name2")){
                            attrSetLatch.countDown();
                        }

                        if (type.equals("del")){
                            attrDelLatch.countDown();
                        }

                        if (type.equals("clear")){
                            attrClearLatch.countDown();

                            channel.channelLeave();
                        }

                    }

                    @Override
                    public void onMessageChannelReceive(Signal.LoginSession session, Signal.LoginSession.Channel channel, String account, int uid, String msg) {
                        if (msg.equals("msg1")){
                            channelMsgLatch1.countDown();
                        }
                    }

                    @Override
                    public void onChannelLeaved(Signal.LoginSession session, Signal.LoginSession.Channel channel, int ecode) {
                        session.logout();
                    }
                });
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch2.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }
        });

        // login
        wait_time(loginLatch2, 5000);
        wait_time(joinLatch2, 5000);
        wait_time(joinListLatch2, 5000);
        wait_time(joinAttrLatch2, 5000);

        wait_time(attrSet_name_Latch, 5000);
        wait_time(attrClearLatch, 5000);
        wait_time(attrDelLatch, 5000);
        wait_time(attrSetLatch, 5000);
        wait_time(channelMsgLatch1, 5000);

        wait_time(userJoinLatch, 5000);
        wait_time(userLeaveLatch, 5000);


        // logout
        wait_time(logoutLatch1, 5000);
        wait_time(logoutLatch2, 5000);
    }


}
