package io.agora.signal;

import org.json.simple.JSONObject;
import org.junit.Test;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

/**
 * Created by aa on 13/4/2017.
 */
public class SignalTestInvite {
    public static final String appid = "place appid to here";

    String user1 = "test_invite_user1";
    String user2 = "test_invite_user2";
    String user3 = "test_invite_user3";
    String user4 = "test_invite_user4";
    String user5 = "test_invite_user5";

    String channelName = "testInviteRoom";

    public static void wait_time(CountDownLatch x, int tInMS){
        try {
            x.await(tInMS, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
        }
        assertEquals(x.getCount(), 0);
    }

    /**
     * test :
     *      API login
     *      API logout
     *          onLoginSuccess
     *          onLogout
     */
    @Test
    public void testInvite(){

        Signal sig = new Signal(appid);

//        System.out.println();
//        System.out.println();

        final CountDownLatch loginLatch1 = new CountDownLatch(1);
        final CountDownLatch logoutLatch1 = new CountDownLatch(1);

        final CountDownLatch loginLatch2 = new CountDownLatch(1);
        final CountDownLatch logoutLatch2 = new CountDownLatch(1);

        final CountDownLatch loginLatch3 = new CountDownLatch(1);
        final CountDownLatch logoutLatch3 = new CountDownLatch(1);

        final CountDownLatch loginLatch4 = new CountDownLatch(1);
        final CountDownLatch logoutLatch4 = new CountDownLatch(1);

        final CountDownLatch inviteAck1 = new CountDownLatch(1);
        final CountDownLatch inviteAck2 = new CountDownLatch(1);
        final CountDownLatch inviteAck3 = new CountDownLatch(1);

        final CountDownLatch inviteAccept1 = new CountDownLatch(1);
        final CountDownLatch inviteAccept2 = new CountDownLatch(1);
        final CountDownLatch inviteRefuse3 = new CountDownLatch(1);

        final CountDownLatch inviteEnd1 = new CountDownLatch(2);
        final CountDownLatch inviteEnd2 = new CountDownLatch(2);
//        final CountDownLatch inviteEnd3 = new CountDownLatch(1);

        final CountDownLatch inviteReceived1 = new CountDownLatch(1);
        final CountDownLatch inviteReceived2 = new CountDownLatch(1);
        final CountDownLatch inviteReceived3 = new CountDownLatch(1);

        final CountDownLatch inviteFailed1 = new CountDownLatch(1);

        Signal.LoginSession s1 = sig.login(user1, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch1.countDown();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch1.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }
        });

        Signal.LoginSession s2 = sig.login(user2, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch2.countDown();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch2.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }

            @Override
            public void onInviteReceived(Signal.LoginSession session, Signal.LoginSession.Call call) {
                inviteReceived1.countDown();
                call.channelInviteAccept();

                call.setCallback(new Signal.CallCallback(){
                    @Override
                    public void onInviteEndByPeer(Signal.LoginSession session,Signal.LoginSession.Call call, String extra) {
                        inviteEnd1.countDown();
                        session.logout();
                    }
                });
            }
        });

        Signal.LoginSession s3 = sig.login(user3, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch3.countDown();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch3.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }

            @Override
            public void onInviteReceived(Signal.LoginSession session, Signal.LoginSession.Call call) {
                inviteReceived2.countDown();
                call.channelInviteAccept();
                call.channelInviteEnd();

                call.setCallback(new Signal.CallCallback(){
                    @Override
                    public void onInviteEndByMyself(Signal.LoginSession session, Signal.LoginSession.Call call, String extra) {
                        inviteEnd2.countDown();
                        session.logout();
                    }
                });
            }
        });

        Signal.LoginSession s4 = sig.login(user4, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch4.countDown();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch4.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }

            @Override
            public void onInviteReceived(Signal.LoginSession session, Signal.LoginSession.Call call) {
                inviteReceived3.countDown();
                call.channelInviteRefuse();
                session.logout();
            }
        });

        wait_time(loginLatch1, 5000);
        wait_time(loginLatch2, 5000);
        wait_time(loginLatch3, 5000);

        // invite 2, 3
        JSONObject extra = new JSONObject();
        extra.put("hello", "world");
        s1.channelInviteUser2(channelName, user2, extra.toJSONString(), new Signal.CallCallback(){
            @Override
            public void onInviteReceivedByPeer(Signal.LoginSession session, Signal.LoginSession.Call call) {
                inviteAck1.countDown();
            }

            @Override
            public void onInviteAcceptedByPeer(Signal.LoginSession session, Signal.LoginSession.Call call, String extra) {
                inviteAccept1.countDown();
                call.channelInviteEnd();
            }

            @Override
            public void onInviteEndByMyself(Signal.LoginSession session, Signal.LoginSession.Call call, String extra) {
                inviteEnd1.countDown();
            }
        });
        s1.channelInviteUser2(channelName, user3, extra.toJSONString(), new Signal.CallCallback(){
            @Override
            public void onInviteReceivedByPeer(Signal.LoginSession session, Signal.LoginSession.Call call) {
                inviteAck2.countDown();
            }

            @Override
            public void onInviteAcceptedByPeer(Signal.LoginSession session, Signal.LoginSession.Call call, String extra) {
                inviteAccept2.countDown();
                call.channelInviteEnd();
            }

            @Override
            public void onInviteEndByPeer(Signal.LoginSession session, Signal.LoginSession.Call call, String extra) {
                inviteEnd2.countDown();
            }

        });
        s1.channelInviteUser2(channelName, user4, extra.toJSONString(), new Signal.CallCallback(){
            @Override
            public void onInviteReceivedByPeer(Signal.LoginSession session, Signal.LoginSession.Call call) {
                inviteAck3.countDown();
            }

            @Override
            public void onInviteRefusedByPeer(Signal.LoginSession session, Signal.LoginSession.Call call, String extra) {
                inviteRefuse3.countDown();
            }
        });
        s1.channelInviteUser2(channelName, user5, extra.toJSONString(), new Signal.CallCallback(){
            @Override
            public void onInviteFailed(Signal.LoginSession session, Signal.LoginSession.Call call, int ecode) {
                inviteFailed1.countDown();
            }
        });

        wait_time(inviteReceived1, 5000);
        wait_time(inviteReceived1, 5000);
        wait_time(inviteReceived1, 5000);

        wait_time(inviteAck1, 5000);
        wait_time(inviteAck2, 5000);
        wait_time(inviteAck3, 5000);

        wait_time(inviteAccept1, 5000);
        wait_time(inviteAccept2, 5000);
        wait_time(inviteRefuse3, 5000);

        wait_time(inviteEnd1, 5000);
        wait_time(inviteEnd2, 5000);

        wait_time(inviteFailed1, 25000);

        s1.logout();

        wait_time(logoutLatch1, 5000);
        wait_time(logoutLatch2, 5000);
        wait_time(logoutLatch3, 5000);
    }

}
