package io.agora.signal;

import org.junit.Test;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

/**
 * Created by aa on 13/4/2017.
 */
public class SignalTestLogin {
    public static final String appid = "place appid to here";


    String user1 = "test_login_user1";
    String user2 = "test_login_user2";

    public static void wait_time(CountDownLatch x, int tInMS){
        try {
            x.await(tInMS, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
        }
        assertEquals(x.getCount(), 0);
    }

    /**
     * test :
     *      API login
     *      API logout
     *          onLoginSuccess
     *          onLogout
     */
    @Test
    public void testLogin(){

//        System.out.println();
//        System.out.println();

        final CountDownLatch loginLatch = new CountDownLatch(1);
        final CountDownLatch logoutLatch = new CountDownLatch(1);

        Signal sig = new Signal(appid);
        sig.login(user1, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch.countDown();

                session.logout();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }
        });

        wait_time(loginLatch, 5000);
        wait_time(logoutLatch, 5000);
    }

    /**
     * test :
     *      API login
     *      API logout
     *      API messageInstantSend
     *          onLoginSuccess
     *          onLogout
     */
    @Test
    public void testInstanstMsg(){
//        System.out.println();
//        System.out.println();

        final CountDownLatch loginLatch1 = new CountDownLatch(1);
        final CountDownLatch logoutLatch1 = new CountDownLatch(1);

        final CountDownLatch loginLatch2 = new CountDownLatch(1);
        final CountDownLatch logoutLatch2 = new CountDownLatch(1);

        final CountDownLatch msgLatch1 = new CountDownLatch(1);
        final CountDownLatch msgLatch2 = new CountDownLatch(1);

        Signal sig = new Signal(appid);

        Signal.LoginSession s1 = sig.login(user1, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch1.countDown();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch1.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }
        });

        Signal.LoginSession s2 = sig.login(user2, "_no_need_token", new Signal.LoginCallback() {
            @Override
            public void onLoginSuccess(final Signal.LoginSession session, int uid) {
//                System.out.println("cb login success");
                loginLatch2.countDown();
            }

            @Override
            public void onLogout(Signal.LoginSession session, int ecode) {
//                System.out.println("cb logout");
                logoutLatch2.countDown();
                assertEquals(ecode, Signal.LOGOUT_E_USER);
            }

            @Override
            public void onMessageInstantReceive(Signal.LoginSession session, String account, int uid, String msg) {
//                System.out.println("cb msg received");
                msgLatch2.countDown();
                session.logout();
            }
        });

        // login

        wait_time(loginLatch1, 5000);
        wait_time(loginLatch2, 5000);

        s1.messageInstantSend(user2, "hello world", new Signal.MessageCallback(){
            @Override
            public void onMessageSendSuccess(Signal.LoginSession session) {
//                System.out.println("cb msg send success");
                msgLatch1.countDown();
                session.logout();
            }

            @Override
            public void onMessageSendError(Signal.LoginSession session, int ecode) {
                assertFalse(true);
            }
        });

        // msg
        wait_time(msgLatch1, 5000);
        wait_time(msgLatch2, 5000);

        // logout
        wait_time(logoutLatch1, 5000);
        wait_time(logoutLatch2, 5000);
    }

}
